const { createClient } = require("redis");
const uuid = require("uuid");

;(function(global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    global.adapter = factory();
}(this, (function() {

    const client = createClient({
        socket: {
            host: 'redis',
            port: 6379
        },
    });

    // Connect to the redis server
    client.connect();

    const get = async function (guid, auth) {
        let config = await client.get(guid);
        if (! config) {
            return false;
        }
        return JSON.parse(config);
    }

    const load = async function (guid, auth) {
        let config = await get(guid, auth);
        if (! config) {
            return false;
        }
        // Make sure the owner is sent to the frontend
        config.spreadsheet.user_id = config.user_id
        return config.spreadsheet;
    }

    const create = async function (guid, config, query) {
        if (! guid) {
            guid = uuid.v4();
        }

        const result = await client.exists(guid);
        if (! result) {
            if (typeof(config) === 'object') {
                config = JSON.stringify(config);
            }

            // Create a new spreadsheet
            await client.set(guid, config);
        }

        return guid;
    }

    const destroy = async function (guid, query) {
        return await client.del(guid);
    }

    const change = async function (guid, changes) {
        // Get the configuration from the cache
        let config = changes.instance.getConfig();
        // Save that on the redis
        return await client.set(guid, JSON.stringify(config));
    }

    const beforeChange = async function (guid, changes) {
    }

    /**
     * Create a plugin object
     */
    let Extension = function(options) {}

    Extension.license = function(license, options) {}

    Extension.get = get;
    Extension.load = load;
    Extension.create = create;
    Extension.destroy = destroy;
    Extension.change = change;
    Extension.beforeChange = beforeChange;

    return Extension;
})));
